

//********************************************************************
//**** toscope : Osciloscopio para Unix Sound System *****************
//********************************************************************
//**** Basado en OSCOPE ( Ver README ) *******************************
//********************************************************************
//**** Txino - Enero 1998 ********************************************
//********************************************************************
//**** Modified: March 2000 ******************************************
//********************************************************************



#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

#include "toscope.h"		/* program defaults */


//*****************************************************************


/* abort and show system error if given ioctl status is bad */
void oscope_dev::check_status(int status)
{
  if (status < 0) {
    perror("oscope_dev: error from sound device ioctl.");    
    exit(1);
  }
}


/* Inicializar valores y reservar memoria */
void oscope_dev::crea(int r, int b, int c)
{

   fragment_size=DEFAULT_OSCOPE_FRAGMENT_SIZE;
   rate=r;
   bps=b;
   channels=c;

   scanline_size=512;

   buffer=(byte *) new byte[fragment_size*channels];
   if (buffer==NULL)
    {
      perror("Out of memory !!!\n"); 
      exit(1);
    }
  
//   int cont;
//   for (cont=0;cont<fragment_size*channels;cont++) buffer[cont]=128;
}


void oscope_dev::close_dev()
{
  close(dev_h);
}


/* attempt to change sample rate and return actual sample rate set */
void oscope_dev::set_rate(int r)
{ 
  int ra=r;
  check_status(ioctl(dev_h, SOUND_PCM_SYNC, 0));
  check_status(ioctl(dev_h, SOUND_PCM_WRITE_RATE, &ra));
  check_status(ioctl(dev_h, SOUND_PCM_READ_RATE, &ra));

  rate=ra;
}

/* [re]set the sound card, and return actual sample rate */
void oscope_dev::reset_dev()
{
  /* set mono/stereo */
  check_status(ioctl(dev_h, SOUND_PCM_WRITE_CHANNELS, &channels));
  check_status(ioctl(dev_h, SOUND_PCM_READ_CHANNELS, &channels));

  /* set 8-bit samples */
  check_status(ioctl(dev_h, SOUND_PCM_WRITE_BITS, &bps));
  check_status(ioctl(dev_h, SOUND_PCM_READ_BITS, &bps));

  /* set sampling rate */
  set_rate(rate);
}

boolean oscope_dev::open_dev()
{
  /* open DSP device for read */
  /* we try a few times in case someone else is using device (FvwmAudio) */

  int i=5;
  while ((dev_h = open(DEFAULT_OSCOPE_DEV, O_RDONLY)) < 0 && i > 0) {
    perror("oscope_dev: cannot open /dev/dsp, retrying...");
    sleep(1);
  }
  
  if (dev_h<0) {
    perror("oscope_dev: cannot open /dev/dsp");
    return FALSE;
  }

  int arg = 0x0002000A;
  ioctl(dev_h, SNDCTL_DSP_SETFRAGMENT, &arg);

  ioctl(dev_h, SNDCTL_DSP_GETBLKSIZE, &arg);
  printf("Fragment Size= %d\n",arg);

  //check_status(ioctl(dev_h, SOUND_PCM_SUBDIVIDE, &dma_sub));
  //check_status(ioctl(dev_h, SNDCTL_DSP_SETFRAGMENT,&arg));

  return TRUE;
}


void oscope_dev::flush_dev()
{
  check_status(ioctl(dev_h, SNDCTL_DSP_RESET));
}


//*************************************************************************

/* get data from sound card, return value is whether we triggered or not */
boolean oscope_dev::read_frame()
{
  int i=1;

  // flush the sound card's buffer
  //flush_dev();

  read(dev_h, buffer,fragment_size*2);

  switch (trig_mode)
   {
    case OSCOPE_NO_TRIG: break;

    case OSCOPE_DESC_TRIG:
	while (i++<(fragment_size-scanline_size) &&
	      (buffer[(i-1)*2+trig_chan]<trig_level || buffer[i*2+trig_chan]>=trig_level));
	break;

    case OSCOPE_ASC_TRIG:
	while (i++<(fragment_size-scanline_size) &&
	      (buffer[(i-1)*2+trig_chan]>=trig_level || buffer[i*2+trig_chan]<trig_level));
	break;

   }


  if (i>fragment_size-scanline_size)	// Si no ocurre el disparo ...	
   {
     trig_pos=0;
     return FALSE;
   }

  trig_pos=i;

  return TRUE;
}


float oscope_dev::get_sample(int chan,int n)
{
  int pos=(trig_pos+n)*2+chan;
  if (pos>fragment_size*2) return 0;

  if (bps==8)
   {
    return ((float)((unsigned char *)buffer)[pos])/127.0-1.0;
   }
  else if (bps==16)
   {
     return (float)(((signed short *)buffer)[pos])/32768-1.0;
     //return ((float)(((signed short *)buffer)[n*2+chan])-8192)/32768.0;
   }
  else return 0;
}

